<?php

class Controller_Api_V0_Student extends Controller_Handler_Api
{
    private function normalizeRecordBookData($data)
    {
        /*
         * $data = [ +'externalID' => $this->request->query('id')
                     + 'planExternalID' => $this->request->query('planId')
                    , 'subdivisionExternalID' => $this->request->query('subdivision')
                    , 'status' => $this->request->query('status')
                    , 'level' => $this->request->query('level')
                    , 'form' => $this->request->query('form')
                    , 'speciality' => $this->request->query('speciality')
                    , 'grade' => $this->request->query('grade')
                    , 'group' => $this->request->query('group')
                ];
         */
        if ( !isset($data->id) && isset($data->externalID) ) {
            $ID = Model_RecordBook::withExternalID($data->externalID);
            $data->id = $ID;
        }

        if ( !isset($data->studentID) ) {
            if ( isset($data->studentExternalID) ) {
                $studentID = Model_Student::withExternalID($data->studentExternalID);
                if ($studentID === -1) {
                    $this->badRequestError( $this->makeErrorMsg($data, 'Не найден студент по данному идентификатору.'));
                }

                $data->studentID = $studentID;
            } else {
                $this->badRequestError( $this->makeErrorMsg($data, 'Не указан идентификатор студента.'));
            }
        }

        if ( !isset($data->planID) ) {
            if ( isset($data->planExternalID) ) {
                $planID = Model_Plan::withExternalID($data->planExternalID);

                $data->planID = $planID;
            } else {
                $this->badRequestError( $this->makeErrorMsg($data, 'Не указан идентификатор учебного плана.'));
            }
        }

        if ( !isset($data->facultyID) ) {
            if (  isset($data->facultyExternalID) ) {
                $depid = $this->getFacultyIDByExternalID($data->facultyExternalID);
                if ( $depid === NULL ) {
                    $this->badRequestError( $this->makeErrorMsg($data, 'Ошибка в коде справочника факультета.') );
                }
                $data->facultyID = $depid;
            } else {
                $this->badRequestError( $this->makeErrorMsg($data, 'Не указано подразделение.'));
            }
        }

//print_r($data);
//        echo "\n\n";

        return $data;
    }

    private function normalizeStudentData($data) {
        if ( !isset($data->id) && isset($data->externalID) ) {
            $ID = Model_Student::withExternalID($data->externalID);
            $data->id = $ID;
        }

        if ( !isset($data->departmentID) ) {
            if (  isset($data->departmentExternalID) ) {
                $depid = $this->getDepIDByExternalID($data->departmentExternalID);
                if ( $depid === NULL ) {
                    $this->badRequestError( $this->makeErrorMsg($data, 'Ошибка коде справочника подразделения.') );
                }
                $data->departmentID = $depid;
            } elseif (  isset($data->departmentName) ) {
                $depid = $this->getDepIDByName($data->departmentName);
                if ( $depid === NULL ) {
                    $this->badRequestError( $this->makeErrorMsg($data, 'Ошибка в имени подразделения.') );
                }
                $data->departmentID = $depid;
            } else {
                $this->badRequestError( $this->makeErrorMsg($data, 'Не указано подразделение.'));
            }
        }

        echo $data->firstName.' '.$data->secondName.' '.$data->lastName.' $data->id: '.$data->id."\n";
        if ( !isset($data->id) || ($data->id == -1) ) {
            echo "-- Try to find by name !\n";
            $foundedList = Model_Teachers::search([$data->firstName, $data->secondName, $data->lastName], 0, 0);
            if ( count($foundedList) === 1 ) {
                echo '--- Founded by name id: '.$foundedList[0]['ID']."!\n";
                $data->id = $foundedList[0]['ID'];
            }
        }

        if ( !isset($data->jobPositionID) && isset($data->jobPositionName) ) {
            $jpID = Model_Faculties::getJobPositionIdByName($data->jobPositionName);
            if ( $jpID === NULL ) {
                $jpID = Model_Faculties::createJobPosition($data->jobPositionName);
            }
            $data->jobPositionID = $jpID;
        } elseif( !isset($data->jobPositionName) ) {
            $this->badRequestError( $this->makeErrorMsg($data, 'Не указана должность.') );
        }

        switch ($data->status) {
            case 'Работает':
                $data->status = true;
                break;
            case 'Уволен':
                $data->status = false;
                break;
            default:
                $this->badRequestError( $this->makeErrorMsg($data, 'Указан некорректный статус.') );
        }

        return $data;
    }

    private function createRecordBook($data) {
        try {
            $recordbook = Model_RecordBook::make()
                ->studentID($data->studentID)
                ->planID($data->planID)
                ->semesterID(Model_System::$SemesterID);

            if ( isset($data->externalID) ) {
                $recordbook->externalID($data->externalID);
            }

            $recordbook = $recordbook->create();
        } catch (InvalidArgumentException $e) {
            $this->badRequestError( $this->makeErrorMsg($data, $e->getMessage()) );
        }
        return ['ID' => $recordbook->ID];
    }

    private function updateRecordBook($data) {
        try {
//            Model_RecordBook::with($data->id)->changeInfo();
        } catch (InvalidArgumentException $e) {
            $this->badRequestError( $this->makeErrorMsg($data, $e->getMessage()) );
        }
    }

    public function action_get_index() {
        $this->response->body( '{ what: get index }' );

        return '{ what: get index }'; //$this->response;
    }

    /**
     * @api {put} api/v0/student/recordBook Add new record book(s)
     * @apiName Add new record book
     * @apiGroup Record books
     * @apiVersion 0.1.2
     * @apiParam {String} token Api key
     * @apiParam {String} [batch] Flag about massive addition
     * @apiParam {String} firstName
     * @apiParam {String} secondName
     * @apiParam {String} lastName
     * @apiParam {String} departmentName
     * @apiParam {String} [departmentID]
     * @apiParam {String} jobPositionName
     * @apiParam {String} [jobPositionID]
     * @apiParam {String} status
     */
    public function action_put_recordBook() {
        try {
            if ( $this->request->query('batch') !== NULL ) {
                $data = json_decode($this->request->body());

                $res = [];
                foreach ($data as $item) {
                    $data = $this->normalizeRecordBookData($item);
                    $res[] = $this->createRecordBook($data);
                }
            } else {
                $data = [ 'externalID' => $this->request->query('id')
                    , 'planExternalID' => $this->request->query('planId')
                    , 'subdivisionExternalID' => $this->request->query('subdivision')
                    , 'status' => $this->request->query('status')
                    , 'level' => $this->request->query('level')
                    , 'form' => $this->request->query('form')
                    , 'speciality' => $this->request->query('speciality')
                    , 'grade' => $this->request->query('grade')
                    , 'group' => $this->request->query('group')
                ];

                $data = $this->normalizeRecordBookData((object)$data);
                $res[] = $this->createRecordBook($data);
            }
        } catch (Exception $e) {
            $this->badRequestError($e->getMessage());
        }

        return $res;
    }

    /**
     * @api {put} api/v0/student Add new student(s)
     * @apiName Add new student
     * @apiGroup Student
     * @apiVersion 0.1.0
     * @apiParam {String} token Api key
     * @apiParam {String} [batch] Flag about massive addition
     * @apiParam {String} firstName
     * @apiParam {String} secondName
     * @apiParam {String} lastName
     * @apiParam {String} departmentName
     * @apiParam {String} [departmentID]
     * @apiParam {String} jobPositionName
     * @apiParam {String} [jobPositionID]
     * @apiParam {String} status
     */
    public function action_put_index() {
        try {
            if ( $this->request->query('batch') !== NULL ) {
                $data = json_decode($this->request->body());

                $res = [];
                foreach ($data as $item) {
                    $data = $this->normalizeStudentData($item);
                    $res[] = $this->createRecordBook($data);
                }
            } else {
                $data = [ 'firstName' => $this->request->query('firstName')
                    , 'secondName' => $this->request->query('secondName')
                    , 'lastName' => $this->request->query('lastName')
                    , 'departmentName' => $this->request->query('departmentName')
                    , 'departmentID' => $this->request->query('departmentID')
                    , 'jobPositionName' => $this->request->query('jobPositionName')
                    , 'jobPositionID' => $this->request->query('jobPositionID')
                    , 'status' => $this->request->query('status')];

                $data = $this->normalizeStudentData((object)$data);
                $res[] = $this->createRecordBook($data);
            }
        } catch (Exception $e) {
            $this->badRequestError($e->getMessage());
        }

        return $res;
    }

    public function action_get_list_by_name() {
        return '{ what: nothing }';
    }
}
